/*
    PlotAxes.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

//class to plot an array using the stem plot

class PlotAxes
{
  //Position and dimensions of the plot
  int xPos,yPos;
  int widthPlot,heightPlot;
  
  //Data specifications
  float dataMin,dataMax;
  int nMin,nMax;
  
  //Scale and origin of the axes
  float T,scaleY;                    /* T - Processing distance per unit n;    scaleY - data range per unit Processing heightPlot  */
  int originX,originY;
  
  //Appearance attributes
  color backgroundColor,gridColor;
  color plotColor,flowerColor;
  PFont font;
  
  PlotAxes() {}
  
  PlotAxes(int xPos_,int yPos_,int widthPlot_,int heightPlot_,PFont font_)
  {
    //Storing construction values for later reference
    xPos=xPos_;
    yPos=yPos_;
    widthPlot=widthPlot_;
    heightPlot=heightPlot_;
    backgroundColor=color(255);
    gridColor=color(200,200,255);
    plotColor=color(255,0,0);
    flowerColor=color(255,255,0);
    font=font_;
    
    //Drawing the border of the plot on the sketch
    stroke(gridColor);
    fill(backgroundColor);
    rect(xPos,yPos,widthPlot,heightPlot);
  }
  
  void stem(DTDataPair data)
  {
    //Drawing the border of the plot on the sketch
    noStroke();
    fill(backgroundColor);
    rect(xPos-100,yPos-10,widthPlot+120,heightPlot+40);
    stroke(gridColor);
    rect(xPos,yPos,widthPlot,heightPlot);
    
    //Storing data specifications for later reference
    if (data.x.length==0)  return;
    dataMin=min(data.x);
    dataMax=max(data.x);
    nMin=min(data.n);
    nMax=max(data.n);

    determineAxes();
    calibrateAxes();

    //Plot the given values 
    pushMatrix();
    translate(originX,originY);
    stroke(plotColor);
    float t;
    
    for (int i=0;i<=nMax-nMin;i++)
    {
      t=(nMin+i)*T;
      strokeWeight(2);
      line(t,0,t,-data.x[i]/scaleY);
      fill(flowerColor);
      ellipse(t,-data.x[i]/scaleY,6,6);
      strokeWeight(1);
    }
    popMatrix();
  }
  
  void determineAxes()
  {
    //Determine X axis scale and origin
    float range=nMax-nMin;
    range= range==0?2:range;
      
    T=float(widthPlot)/range;
    originX=xPos-int(nMin*T);
    
    //Determine Y axis scale and origin
    float dataMin_=dataMin<0?dataMin:0;
    float dataMax_=dataMax>0?dataMax:0;
    scaleY=(dataMax_-dataMin_)/heightPlot;
    scaleY=scaleY==0?2:scaleY;
    originY=yPos+heightPlot+int(dataMin_/scaleY);
  }
  
  void calibrateAxes()
  {   
    calibrateXAxis();
    calibrateYAxis();
  }
  
  void calibrateXAxis()
  {
    //Determine gapN and gapDistance
    int gapN=round(50/T);
    if (gapN==0) gapN=1;
    float gapDistance=gapN*T;
    
    //Calibrate the x axis
    pushMatrix();
    translate(originX,yPos+heightPlot);
    fill(0);
    textFont(font,14);
    textAlign(CENTER);

    for (int i=nMin/gapN;i<=nMax/gapN;i++)
    {
      stroke(0);
      line(i*gapDistance,5,i*gapDistance,0);
      text(i*gapN,i*gapDistance,20);
      stroke(gridColor);
      line(i*gapDistance,0,i*gapDistance,-heightPlot);
    }
    popMatrix();
  }
  
  void calibrateYAxis()
  {
    //Determine gapData and gapHeight
    float gapData=50*scaleY;
    float gapHeight;
    int exponent=0;
    while (gapData<1)
    {
      gapData=gapData*10;
      exponent--;
    }
    while (gapData>=10)
    {
      gapData=gapData/10;
      exponent++;
    }
    gapData=round(gapData)*pow(10,exponent);
    gapHeight=gapData/scaleY;
    
    //Calibrate the y axis
    pushMatrix();
    translate(xPos,originY);
    fill(0);
    textFont(font,14);
    textAlign(RIGHT);
    
    float dataMin_=dataMin<0?dataMin:0;
    float dataMax_=dataMax>0?dataMax:0;
    for (int i=int(dataMin_/gapData);i<=dataMax_/gapData;i++)
    {
      stroke(0);
      line(-5,-i*gapHeight,0,-i*gapHeight);
      text(i*gapData,-8,-i*gapHeight);
      stroke(gridColor);
      line(0,-i*gapHeight,widthPlot,-i*gapHeight);
    }
  
    popMatrix();
  }
}
