/*
    SignalAdditionMultiplication800x600.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

/* Program to illustrate the concepts of signal addition and multiplication
    Done as part of Signals and Systems Assignment, fourth semester, BE ECE
*/

PFont f;

DTDataPair input1,input2;

PlotAxes inputAxes1,inputAxes2,sumAxes,productAxes;

Button impulse1,step1,sine1,square1,exponential1,absolute1;
Button impulse2,step2,sine2,square2,exponential2,absolute2;
Button currentSignal1,currentSignal2;

SignalGenerator agilent;

void setup()
{
  //initialize font
  f=loadFont("WenQuanYiMicroHeiMono-15.vlw");
  
  //initialize sketch
  size(800,550);
  background(255);
  smooth();
  
  //initialize plots, signal generator
  inputAxes1=new PlotAxes(200,50,180,180,f);
  inputAxes2=new PlotAxes(500,50,180,180,f);
  sumAxes=new PlotAxes(200,300,180,180,f);
  productAxes=new PlotAxes(500,300,180,180,f);
  agilent=new SignalGenerator();
  
  
  //initialize buttons for input signal 1
  impulse1=new Button(10,50,80,30,"Impulse",f);
  step1=new Button(10,100,80,30,"Step",f);
  sine1=new Button(10,150,80,30,"Sine",f);
  square1=new Button(10,200,80,30,"Square",f);
  exponential1=new Button(10,250,80,30,"Exp",f);
  absolute1=new Button(10,300,80,30,"Abs",f);
  
  //initialize buttons for input signal 2
  impulse2=new Button(710,50,80,30,"Impulse",f);
  step2=new Button(710,100,80,30,"Step",f);
  sine2=new Button(710,150,80,30,"Sine",f);
  square2=new Button(710,200,80,30,"Square",f);
  exponential2=new Button(710,250,80,30,"Exp",f);
  absolute2=new Button(710,300,80,30,"Abs",f);
  
  //set initial signal
  input1=agilent.unitImpulseSequence(-10,10);
  input2=agilent.unitImpulseSequence(-10,10);
  currentSignal1=impulse1;
  currentSignal2=impulse2;
  updateSignal1(impulse1);
  updateSignal2(impulse2);
  labelPlots();
}

void draw()
{
}

void mousePressed()
{
  //check for press of input signal 1 buttons
  if (impulse1.pressed())  updateSignal1(impulse1);
  else if (step1.pressed()) updateSignal1(step1);
  else if (sine1.pressed()) updateSignal1(sine1);
  else if (square1.pressed()) updateSignal1(square1);
  else if (exponential1.pressed()) updateSignal1(exponential1);
  else if (absolute1.pressed()) updateSignal1(absolute1);
  
  //check for press of input signal 2 buttons
  if (impulse2.pressed())  updateSignal2(impulse2);
  else if (step2.pressed()) updateSignal2(step2);
  else if (sine2.pressed()) updateSignal2(sine2);
  else if (square2.pressed()) updateSignal2(square2);
  else if (exponential2.pressed()) updateSignal2(exponential2);
  else if (absolute2.pressed()) updateSignal2(absolute2);
}

void updateSignal1(Button newSignal)
{
  //unlock previously selected signal
  currentSignal1.locked=false;
  currentSignal1.update();
  
  //lock currently selected signal
  currentSignal1=newSignal;
  currentSignal1.locked=true;
  
  //generate the input signal
  if (newSignal==impulse1)  input1=agilent.unitImpulseSequence(-10,10);
  if (newSignal==step1)  input1=agilent.unitStepSequence(-10,10);
  if (newSignal==sine1)  input1=agilent.sineSequence(-10,10);
  if (newSignal==square1)  input1=agilent.squareSequence(-10,10,4);
  if (newSignal==exponential1)  input1=agilent.expSequence(-10,10);
  if (newSignal==absolute1) input1=agilent.absoluteSequence(-10,10);
  
  //display input and output signals
  currentSignal1.update();
  inputAxes1.stem(input1);
  addSignals();
  multiplySignals();
}

void updateSignal2(Button newSignal)
{
  //unlock previously selected signal
  currentSignal2.locked=false;
  currentSignal2.update();
  
  //lock currently selected signal
  currentSignal2=newSignal;
  currentSignal2.locked=true;

  //generate the input signal
  if (newSignal==impulse2)  input2=agilent.unitImpulseSequence(-10,10);
  if (newSignal==step2)  input2=agilent.unitStepSequence(-10,10);
  if (newSignal==sine2)  input2=agilent.sineSequence(-10,10);
  if (newSignal==square2)  input2=agilent.squareSequence(-10,10,4);
  if (newSignal==exponential2)  input2=agilent.expSequence(-10,10);
  if (newSignal==absolute2) input2=agilent.absoluteSequence(-10,10);  

  //display input and output signals
  currentSignal2.update();
  inputAxes2.stem(input2);
  addSignals();
  multiplySignals();
}

void addSignals()
{
  DTDataPair sum=new DTDataPair(new int[input1.n.length],new float[input1.n.length]);
  for (int i=0;i<=input1.n.length-1;i++)
  {
    sum.n[i]=input1.n[i];
    sum.x[i]=input1.x[i]+input2.x[i];
  }
  sumAxes.stem(sum);
}

void multiplySignals()
{
  DTDataPair product=new DTDataPair(new int[input1.n.length],new float[input1.n.length]);
  for (int i=0;i<=input1.n.length-1;i++)
  {
    product.n[i]=input1.n[i];
    product.x[i]=input1.x[i]*input2.x[i];
  }
  productAxes.stem(product);
}

void labelPlots()
{
  //clear label area
  noStroke();
  fill(255);
  rect(245,20,100,20);
  rect(540,20,100,20);
  rect(245,270,100,20);
  rect(540,270,100,20);
  
  //print labels
  fill(0);
  textAlign(CENTER,TOP);
  text("x[n]",295,25);
  text("y[n]",595,25);
  text("x[n]+y[n]",295,275);
  text("x[n].y[n]",595,275);
}
